#!/bin/sh
#----------------------------------------------------------------------------
# ipv6-interface-up.sh                                           3.10.4
#
#
# Last Update:  $Id: ipv6-interface-up.sh 28088 2013-07-18 13:33:14Z kristov $
#----------------------------------------------------------------------------

# $1 = file containing IPv6 network data
# $2 = subnet prefix to use
# $3 = valid lifetime of prefix
# $4 = preferred lifetime of prefix

# include IPv4 helper functions (especially log_* and translate_ip_net)
. /etc/boot.d/base-helper
# include IPv6 helper functions (especially fliplocalbit)
. /etc/rc.d/base-helper.ipv6

dhcpdevices=/var/run/dhcpd6.if

netfile="$1"
if [ ! -f "$netfile" ]
then
    log_error "$(basename $0): IPv6 network information file '$netfile' does not exist"
    exit 1
fi

. "$netfile"

netname="IPV6_NET_$(basename "$netfile" .conf)"
net="$2"
netaddr=$(echo $net | cut -d '/' -f 1)
netlen=$(echo $net | cut -d '/' -f 2)

prefix=$(prefixcalc6 $net)
prefixaddr=$(echo $prefix | cut -d '/' -f 1)
prefixvalidlft=$3
prefixpreflft=$4

# lifetime handling: if a prefix lifetime exists, it overrides the
# configuration found in IPV6_NET_x_(VALID|PREF)_LIFETIME
[ -n "$prefixvalidlft" ] && advvalidlft=$prefixvalidlft
[ -n "$prefixpreflft" ]  && advpreflft=$prefixpreflft

# assign defaults
: ${advvalidlft:='86400'}
: ${advpreflft:='14400'}
: ${prefixvalidlft:='forever'}
: ${prefixpreflft:='forever'}

sed -i "s,^ifnet=.*,ifnet=$net," "$netfile"
sed -i "s,^ifnetvalidlft=.*,ifnetvalidlft=$prefixvalidlft," "$netfile"
sed -i "s,^ifnetpreflft=.*,ifnetpreflft=$prefixpreflft," "$netfile"

eval $(grep "^\(HOSTNAME\|DOMAIN_NAME\|OPT_DNS\|DNS_SUPPORT_IPV6\|DNS_LISTEN_N\)=" /etc/rc.cfg)

if [ "$adv" = "yes" ]
then
    {
        if [ $advvalidlft = "forever" ]
        then
            validlifetime=infinity
        else
            validlifetime=$advvalidlft
        fi
        if [ $advpreflft = "forever" ]
        then
            preferredlifetime=infinity
        else
            preferredlifetime=$advpreflft
        fi

        echo "    prefix $prefix {"
        echo "      AdvValidLifetime $validlifetime;"
        echo "      AdvPreferredLifetime $preferredlifetime;"
        echo "    };"
        log_info "$(basename $0): $netname: advertising prefix $prefix on $dev for $advvalidlft/$advpreflft seconds"

        if [ "$adv_dns" = "yes" ]
        then
            echo "    RDNSS $netaddr {"
            echo "    };"
        fi
    } > /var/run/radvd-$dev.conf
    > /var/run/reload.radvd
fi

if [ "$dhcp" = "yes" ]
then
    echo $dev >> "$dhcpdevices"
    {
        echo "subnet6 $prefix {"
        if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" ]
        then
            echo "  option dhcp6.name-servers $netaddr;"
        fi
        echo "}"
    } > /var/run/dhcpd6-$dev.conf
    > /var/run/reload.dhcpd6
fi

if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" -a "$DNS_LISTEN_N" != "0" ]
then
    if [ "$dhcp" = "yes" -o "$adv_dns" = "yes" ]
    then
        mkdir -p /etc/dnsmasq.d
        echo "listen-address=$netaddr" >> /etc/dnsmasq.d/dns_ipv6.conf
        > /var/run/reload.dnsmasq
    fi
fi

ip -6 addr add $net dev $dev valid_lft $prefixvalidlft preferred_lft $prefixpreflft
log_info "$(basename $0): $netname: address $netaddr added to $dev for $prefixvalidlft/$prefixpreflft seconds"

dnsdev=$(echo -n $dev | sed 's/\./-vlan-/;s/:/-alias-/')
echo "$netaddr $canonical $HOSTNAME-$dnsdev.$DOMAIN_NAME $HOSTNAME-$dnsdev" >> /etc/hosts

if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" -a $netlen -eq 64 ]
then
    eval $(grep "^HOST_\(N\|[0-9]\+_\(IP[46]\|IP6_NET\|MAC\)\)=" /etc/rc.cfg)
    for idx in $(seq 1 $HOST_N)
    do
        eval ip6='$HOST_'$idx'_IP6'
        eval ip6net='$HOST_'$idx'_IP6_NET'
        eval mac='$HOST_'$idx'_MAC'
        : ${ip6net:="IPV6_NET_1"}

        if [ "$ip6" = "auto" -a "$ip6net" = "$netname" -a -n "$mac" ]
        then
            eval ip4='$HOST_'$idx'_IP4'
            if translate_ip_net $ip4 HOST_${idx}_IP4
            then
                ip4=$res
            fi

            # compute EUI-64 from MAC
            set -- $(echo -n $mac | cut -d ':' -f 1 | sed 's/^\(.\)\(.\).*$/\1 \2/')
            first=$1
            fliplocalbit $2

            eui="$first$result$(echo -n $mac | cut -d ':' -f 2,3)ff:fe$(echo -n $mac | cut -d ':' -f 4,5)$(echo -n $mac | cut -d ':' -f 6)"
            addr=$(echo -n $prefixaddr | sed "s/0:0:0:0$/$eui/")
            for f in /etc/hosts /etc/hosts.d/hosts.*
            do
                [ -f "$f" ] || continue
                sed -i "s/^\(#*::ffff:$ip4 .*\)/#\1/" $f
                grep "^#\+::ffff:$ip4 " $f | sed "s/^#\+::ffff:$ip4 /$addr /" >> $f
            done
        fi
    done
fi

exit 0
